@@ -24,7 +24,7 @@ namespace py = pybind11;
 using namespace gemmi;
 
 template<typename T>
-void add_grid_base(py::module& m, const char* name) {
+py::class_<GridBase<T>, GridMeta> add_grid_base(py::module& m, const char* name) {
   using GrBase = GridBase<T>;
   using GrPoint = typename GridBase<T>::Point;
 
@@ -56,6 +56,7 @@ void add_grid_base(py::module& m, const char* name) {
     .def("__iter__", [](GrBase& self) { return py::make_iterator(self); },
          py::keep_alive<0, 1>())
     ;
+  return grid_base;
 }
 
 template<typename T>
@@ -89,6 +90,7 @@ void add_grid(py::module& m, const std::string& name) {
     .def_property_readonly("spacing", [](const Gr& self){
         return py::make_tuple(self.spacing[0], self.spacing[1], self.spacing[2]);
     })
+    .def("set_size", &Gr::set_size)
     .def("get_value", &Gr::get_value)
     .def("set_value", &Gr::set_value)
     .def("get_point", &Gr::get_point)
@@ -97,6 +99,7 @@ void add_grid(py::module& m, const std::string& name) {
     .def("get_nearest_point", (GrPoint (Gr::*)(const Position&)) &Gr::get_nearest_point)
     .def("point_to_fractional", &Gr::point_to_fractional)
     .def("point_to_position", &Gr::point_to_position)
+    .def("change_values", &Gr::change_values, py::arg("old_value"), py::arg("new_value"))
     .def("interpolate_value",
          (T (Gr::*)(const Fractional&) const) &Gr::interpolate_value)
     .def("interpolate_value",
@@ -111,17 +114,24 @@ void add_grid(py::module& m, const std::string& name) {
               r(i, j, k) = self.interpolate_value(pos);
             }
     }, py::arg().noconvert(), py::arg())
+    .def("tricubic_interpolation",
+         (double (Gr::*)(const Fractional&) const) &Gr::tricubic_interpolation)
+    .def("tricubic_interpolation",
+         (double (Gr::*)(const Position&) const) &Gr::tricubic_interpolation)
+    .def("tricubic_interpolation_der",
+         (std::array<double,4> (Gr::*)(const Fractional&) const)
+         &Gr::tricubic_interpolation_der)
     .def("copy_metadata_from", &Gr::copy_metadata_from)
     .def("setup_from", &Gr::template setup_from<Structure>,
          py::arg("st"), py::arg("spacing"))
     .def("set_unit_cell", (void (Gr::*)(const UnitCell&)) &Gr::set_unit_cell)
     .def("set_points_around", &Gr::set_points_around,
-         py::arg("position"), py::arg("radius"), py::arg("value"))
+         py::arg("position"), py::arg("radius"), py::arg("value"), py::arg("use_pbc")=true)
     .def("symmetrize_min", &Gr::symmetrize_min)
     .def("symmetrize_max", &Gr::symmetrize_max)
     .def("symmetrize_abs_max", &Gr::symmetrize_abs_max)
     .def("symmetrize_sum", &Gr::symmetrize_sum)
-    .def("asu", &Gr::asu)
+    .def("masked_asu", &Gr::masked_asu, py::keep_alive<0, 1>())
     .def("mask_points_in_constant_radius", &mask_points_in_constant_radius<T>,
          py::arg("model"), py::arg("radius"), py::arg("value"))
     .def("get_subarray",
@@ -137,13 +147,19 @@ void add_grid(py::module& m, const std::string& name) {
               ptr[idx++] = self.get_value(u0 + u, v0 + v, w0 + w);
         return arr;
     })
+    .def("clone", [](const Gr& self) { return new Gr(self); })
     .def("__repr__", [=](const Gr& self) {
         return tostr("<gemmi.", name, '(', self.nu, ", ", self.nv, ", ", self.nw, ")>");
     });
 
   masked_grid
     .def_readonly("grid", &Masked::grid, py::return_value_policy::reference)
-    .def_readonly("mask", &Masked::mask)
+    .def_property_readonly("mask_array", [](const Masked& self) {
+      const Gr& gr = *self.grid;
+      py::array::ShapeContainer shape({gr.nu, gr.nv, gr.nw});
+      py::array::StridesContainer strides({gr.nv * gr.nw, gr.nw, 1});
+      return py::array_t<std::int8_t>(shape, strides, self.mask.data(), py::cast(self));
+    }, py::return_value_policy::reference_internal)
     .def("__iter__", [](Masked& self) { return py::make_iterator(self); },
          py::keep_alive<0, 1>())
     ;
@@ -184,7 +200,9 @@ void add_grid(py::module& m) {
 
   add_grid_base<int8_t>(m, "Int8GridBase");
   add_grid<int8_t>(m, "Int8Grid");
-  add_grid_base<float>(m, "FloatGridBase");
+  add_grid_base<float>(m, "FloatGridBase")
+    .def("calculate_correlation", &calculate_correlation<float>)
+    ;
   add_grid<float>(m, "FloatGrid");
   add_grid_base<std::complex<float>>(m, "ComplexGridBase");
   m.def("interpolate_positions", &interpolate_positions);
@@ -209,7 +227,12 @@ void add_grid(py::module& m) {
     .def("put_mask_on_float_grid", &SolventMasker::put_mask_on_grid<float>)
     .def("set_to_zero", &SolventMasker::set_to_zero)
     ;
+  m.def("interpolate_grid_of_aligned_model2", &interpolate_grid_of_aligned_model2<float>,
+        py::arg("dest"), py::arg("src"), py::arg("tr"),
+        py::arg("dest_model"), py::arg("radius"));
 
+
+  // from blob.hpp
   py::class_<Blob>(m, "Blob")
     .def_readonly("volume", &Blob::volume)
     .def_readonly("score", &Blob::score)
